// Backbone Server
// Nate Hunzaker
//
// Adds a module for server side interaction within Backbone using Connect


//-- Requirements ------------------------------------------------------//

var Backbone   = require('backbone');
    express    = require('express');


//-- Backbone Server ---------------------------------------------------//

Backbone.Server = Backbone.Router.extend({
    
    //-- Default Values ------------------------------------------------//
    
    'port'         : 8000,
    'public'       : 'public',
    'views'        : 'views',
    'view-engine'  : 'jade',
    'routes'       : {},
    'app'          : express.createServer(),    
    
    
    //-- Initialization ------------------------------------------------//
    
    initialize: function(params) {
        
        params || (params = {});
        
        this.routes = params['routes'] || this.routes;
        this.port   = params['port'] || this.port;
        
        this.app.use(express.static(this.public));
        this.app.set('view engine', this['view-engine']);
    },
    
    
    //-- Routing Methods Values ----------------------------------------//
    
    route: function (method, path, action) {
        this.routes[path] = [method, action]
    },
    
    get: function(path, action) {
        this.route('get', path, action);
    },
    
    post: function(path, action) {
        this.route('post', path, action);
    },
    
    put: function(path, action) {
        this.route('put', path, action);
    },
    
    delete: function(path, action) {
        this.route('delete', path, action);
    },
    
    
    //-- Server Methods ------------------------------------------------//
    
    start: function() {   
        
        // Add routes
        for (var i in this.routes) {
            
            var method = this.routes[i][0],
                action = this.routes[i][1];
            
            console.log("Route created for " + i);
            this.app[method](i, action);
        }
        
        this.app.listen(this.port);
        console.log("Backbone Server is listening on port " + this.port);
    }
    
});


module.exports = Backbone;